# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:/bld/qgis_1584898378594/work/python/plugins/db_manager/ui/DlgExportVector.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DbManagerDlgExportVector(object):
    def setupUi(self, DbManagerDlgExportVector):
        DbManagerDlgExportVector.setObjectName("DbManagerDlgExportVector")
        DbManagerDlgExportVector.resize(514, 253)
        self.gridLayout_2 = QtWidgets.QGridLayout(DbManagerDlgExportVector)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.editOutputFile = QtWidgets.QLineEdit(DbManagerDlgExportVector)
        self.editOutputFile.setText("")
        self.editOutputFile.setObjectName("editOutputFile")
        self.gridLayout_2.addWidget(self.editOutputFile, 1, 1, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(DbManagerDlgExportVector)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout_2.addWidget(self.buttonBox, 3, 0, 1, 3)
        self.cboFileFormat = QtWidgets.QComboBox(DbManagerDlgExportVector)
        self.cboFileFormat.setObjectName("cboFileFormat")
        self.gridLayout_2.addWidget(self.cboFileFormat, 0, 1, 1, 2)
        self.label_5 = QtWidgets.QLabel(DbManagerDlgExportVector)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setSizePolicy(sizePolicy)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 1, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(DbManagerDlgExportVector)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName("gridLayout")
        self.chkDropTable = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkDropTable.setObjectName("chkDropTable")
        self.gridLayout.addWidget(self.chkDropTable, 2, 0, 1, 2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.chkSourceSrid = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkSourceSrid.setObjectName("chkSourceSrid")
        self.horizontalLayout.addWidget(self.chkSourceSrid)
        self.editSourceSrid = QtWidgets.QLineEdit(self.groupBox_2)
        self.editSourceSrid.setEnabled(False)
        self.editSourceSrid.setText("")
        self.editSourceSrid.setObjectName("editSourceSrid")
        self.horizontalLayout.addWidget(self.editSourceSrid)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.chkTargetSrid = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkTargetSrid.setObjectName("chkTargetSrid")
        self.horizontalLayout.addWidget(self.chkTargetSrid)
        self.editTargetSrid = QtWidgets.QLineEdit(self.groupBox_2)
        self.editTargetSrid.setEnabled(False)
        self.editTargetSrid.setText("")
        self.editTargetSrid.setObjectName("editTargetSrid")
        self.horizontalLayout.addWidget(self.editTargetSrid)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 2)
        self.chkEncoding = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkEncoding.setObjectName("chkEncoding")
        self.gridLayout.addWidget(self.chkEncoding, 1, 0, 1, 1)
        self.cboEncoding = QtWidgets.QComboBox(self.groupBox_2)
        self.cboEncoding.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cboEncoding.sizePolicy().hasHeightForWidth())
        self.cboEncoding.setSizePolicy(sizePolicy)
        self.cboEncoding.setEditable(False)
        self.cboEncoding.setInsertPolicy(QtWidgets.QComboBox.NoInsert)
        self.cboEncoding.setObjectName("cboEncoding")
        self.gridLayout.addWidget(self.cboEncoding, 1, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBox_2, 2, 0, 1, 3)
        self.btnChooseOutputFile = QtWidgets.QToolButton(DbManagerDlgExportVector)
        self.btnChooseOutputFile.setObjectName("btnChooseOutputFile")
        self.gridLayout_2.addWidget(self.btnChooseOutputFile, 1, 2, 1, 1)
        self.label = QtWidgets.QLabel(DbManagerDlgExportVector)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)

        self.retranslateUi(DbManagerDlgExportVector)
        self.buttonBox.rejected.connect(DbManagerDlgExportVector.reject)
        self.chkSourceSrid.toggled['bool'].connect(self.editSourceSrid.setEnabled)
        self.chkTargetSrid.toggled['bool'].connect(self.editTargetSrid.setEnabled)
        self.chkEncoding.toggled['bool'].connect(self.cboEncoding.setEnabled)
        self.buttonBox.accepted.connect(DbManagerDlgExportVector.accept)
        QtCore.QMetaObject.connectSlotsByName(DbManagerDlgExportVector)
        DbManagerDlgExportVector.setTabOrder(self.chkSourceSrid, self.editSourceSrid)
        DbManagerDlgExportVector.setTabOrder(self.editSourceSrid, self.chkEncoding)
        DbManagerDlgExportVector.setTabOrder(self.chkEncoding, self.cboEncoding)
        DbManagerDlgExportVector.setTabOrder(self.cboEncoding, self.buttonBox)

    def retranslateUi(self, DbManagerDlgExportVector):
        _translate = QtCore.QCoreApplication.translate
        DbManagerDlgExportVector.setWindowTitle(_translate("DbManagerDlgExportVector", "Export to vector file"))
        self.label_5.setText(_translate("DbManagerDlgExportVector", "Save as"))
        self.groupBox_2.setTitle(_translate("DbManagerDlgExportVector", "Options"))
        self.chkDropTable.setText(_translate("DbManagerDlgExportVector", "Replace destination file (if exists)"))
        self.chkSourceSrid.setText(_translate("DbManagerDlgExportVector", "Source SRID"))
        self.chkTargetSrid.setText(_translate("DbManagerDlgExportVector", "Target SRID"))
        self.chkEncoding.setText(_translate("DbManagerDlgExportVector", "Encoding"))
        self.btnChooseOutputFile.setText(_translate("DbManagerDlgExportVector", "…"))
        self.label.setText(_translate("DbManagerDlgExportVector", "Format"))
